@@ -2,12 +2,14 @@ from aws_syncr.amazon.common import AmazonMixin
 from aws_syncr.errors import AwsSyncrError
 from aws_syncr.differ import Differ
 
+from input_algorithms.spec_base import NotSpecified
 import requests
 import boto3
 
 import logging
 import json
 import six
+import re
 
 log = logging.getLogger("aws_syncr.amazon.apigateway")
 
@@ -189,8 +191,13 @@ class ApiGateway(AmazonMixin, object):
 
         new_kwargs = new_integration.put_kwargs(location, self.accounts, self.environment)
         old_kwargs = {} if not old_integration else {"type": old_integration["type"]}
+
+        if old_integration and old_integration.get('requestTemplates'):
+            old_kwargs['requestTemplates'] = old_integration['requestTemplates']
+
         if old_kwargs and old_kwargs['type'] == 'AWS':
             old_kwargs['uri'] = old_integration['uri']
+
         changes = list(Differ.compare_two_documents(old_kwargs, new_kwargs))
 
         # Make sure our integration can be called by apigateway
@@ -227,12 +234,18 @@ class ApiGateway(AmazonMixin, object):
             for _ in self.change("+", "gateway resource integration response", gateway=name, resource=path, method=method, status_code=status_code):
                 resource_id = resources_by_path[path]['id']
                 client.put_integration_response(restApiId=gateway_info['identity'], resourceId=resource_id, httpMethod=method, statusCode=str(status_code)
-                    , responseTemplates = dict((m.content_type, m.template) for m in wanted_integration[status_code])
+                    , responseTemplates = {} if not wanted_integration[status_code] else dict((m.content_type, m.template) for m in wanted_integration[status_code])
                     )
 
         for status_code in for_modification:
-            old = old_integration[status_code]['responseTemplates']
-            new = dict((m.content_type, m.template) for m in wanted_integration[status_code])
+            old = old_integration[status_code].get('responseTemplates', {})
+            for ct, template in old.items():
+                if template is None:
+                    old[ct] = ""
+
+            new = {}
+            if wanted_integration[status_code]:
+                new = dict((m.content_type, m.template) for m in wanted_integration[status_code])
             changes = list(Differ.compare_two_documents(old, new))
 
             old = dict((ct.replace('/', '~1'), v) for ct, v in old.items())
@@ -393,10 +406,10 @@ class ApiGateway(AmazonMixin, object):
             return self.client(gateway_location).get_domain_name(domainName=record)['distributionDomainName']
         raise AwsSyncrError("Please do a sync first!")
 
-    def test_stage(self, gateway_info, location, stage, method, endpoint, sample_event):
+    def test_stage(self, gateway_info, location, stage, method, endpoint, sample_event, desired_output_for_test):
         kwargs = {}
         if sample_event:
-            if not isinstance(sample_event, six.string_types):
+            if not isinstance(sample_event, six.string_types) and sample_event is not NotSpecified:
                 kwargs['data'] = json.dumps(dict(sample_event.items()))
 
         # Find the url to use
@@ -434,6 +447,19 @@ class ApiGateway(AmazonMixin, object):
             # Say we failed if status code isn't 200
             return False
         else:
-            # Say we succeeded otherwise
-            return True
+            # Say we succeeded if we meet the desired_output_for_test
+            if desired_output_for_test and desired_output_for_test is not NotSpecified:
+                if isinstance(desired_output_for_test, six.string_types):
+                    content = res.content.decode('utf-8')
+                    if not re.match(desired_output_for_test, content):
+                        print("content '{0}' does not match pattern '{1}'".format(content, desired_output_for_test))
+                        return False
+
+                else:
+                    content = json.loads(res.content.decode('utf-8'))
+                    if any(content[key] != val for key, val in desired_output_for_test.items()):
+                        print("Not all of the values match our desired output of '{0}'".format(desired_output_for_test))
+                        return False
+
+        return True
 
